/*
 * vcdbuttons.h
 * Copyright (C) 2006 - 2008 Josh Sutherland <OneThirty8@aol.com>
 * This file is part of VC/DC, a free MPEG-2 video player.
 *
 * VC/DC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VC/DC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#ifndef _VCDBUTTONS_H
#define _VCDBUTTONS_H

#include "libvcd.h"
/* Generic play/stop/ffwd buttons */
#define BUTTON_PAUSE 1
#define BUTTON_STOP  2
#define BUTTON_FFWD  3
#define BUTTON_PREV  4
#define BUTTON_NEXT  5

/* VCD remote buttons (return, default, numeric) */ 
#define BUTTON_RET   6
#define BUTTON_DEF   7
#define BUTTON_NUMERIC 8

/* Says we pushed a DC button, not a VCD button (yet). */
#define BUTTON_REMOTE 9

/* Says user tried to use the "Numeric Selection" when it is not available. */
#define BUTTON_NO_NUMBERS 138 /* ;-) */

/* This is a struct to hold the information needed for us to know
  if the user has made a valid navigation request via the DC controller
  and/or the onscreen numerical selection entry system. */
typedef struct
{
  uint8 bsn;  /* Base selection number--this is usually 1 but can be any number up to 99.*/
  uint8 nos;  /* Number of possible SelectionList numerical selections. */
  
  /* The next few are fairly self-explanatory. */
  uint16 prev_ofs;
  uint16 next_ofs;
  uint16 return_ofs;
  uint16 default_ofs; /* This is for a button named 'default,' not what to
  			default to. */
  uint16 ptime; /* Play time in 1/15 seconds.  0x0000 means full item. */
  uint16 atime; /* Autopause delay--figured same as wtime in playlists. */
  int jump_delay; /* Whether to jump immediately (0) or wait until the item is finished. */
  PsdSelectionListDescriptorExtended_t *ExtList; /* List of selection areas. */
  int x, y;	/* For a pointer. */
} RemoteControl_t;
#define BUTTON_VALID 1
#define BUTTON_INVALID 0

#endif /*_VCDBUTTONS_H*/
