/* 
 * debug_overlay.c
 * Copyright (c) 2008 Josh Sutherland.
 * This file is part of VC/DC, a free MPEG-2 video player.
 *
 * VC/DC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VC/DC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/* 
 * This is just a set of functions to use the text-drawing functions
 * from GhettoPlay to give us whatever information we decide we want.
 * Probably not all that useful if you have a coder's cable/BBA, but
 * it gives me access to whatever info I want.
 *
 * This file is something I threw together in about 45 minutes after
 * deciding the method I was using in VC/DC v0.1 and v0.1a really sucked.
 * I don't really care what you do with the code in this file, but
 * if you find it useful I'd appreciate it if you leave the copyright
 * notice intact.  Also, if you got this file as a part of a GPL-licensed
 * software package (eg, VC/DC), you must follow the GPL.  If you get it
 * from my page at http://www.storm-studios.net/onethirty8 it will be identical
 * except for the GPL notification above.
*/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "gp.h"
   
   /*So, figure a character is 12x24 pixels.  Also figure there is a margin
     that we can't see.  So, let's say we'll start at x=5, y=60.
     
     480-60 gives us 420 for the bottom of the last line, so 396 for the top.
     
     396-60 = 336.
     
     336/24 = 14 lines.
     
     Line width is 635-5 (ie, start at pixel 5, end at pixel 635) - 12 = 
     630-12 = 618.
     618/12 = 51.5, so say 51 so we don't get cut off.
     so we'll define LINEWIDTH as 51.
     
   */
#define LINEWIDTH 51
   
static struct debug_line_s{
   char line[LINEWIDTH];
   int used;
   char identifier[LINEWIDTH];
} debug_line[14];

static const char * blankid = 
{
  "                         \0"
};
   
static int getnewline(char * noteid)
{
  int i;
    
  for (i=0; i < 14; i++)
  {  
    if (!(strcmp(noteid, debug_line[i].identifier) ) ) {
     /* Try to keep displaying things in the same place. */
      return i;
    } 
  }  
  for (i=0; i < 14; i++)
  {
    if (debug_line[i].used == 0)
    {
      debug_line[i].used = 1;
        sprintf (debug_line[i].line, blankid);
      if (!(noteid==NULL))
        sprintf (debug_line[i].identifier, noteid);
      else
        sprintf (debug_line[i].identifier, blankid);

      return i;
    }  
  }
  return 14;
}

/* If we're going to overwrite the screen, or we want to append a line
  to an existing note, we call this to get a second line for our note,
  if a line is available. */
static int addline(char * noteid)
{
int i, last_line, free_lines;
free_lines = 0; last_line = 0;
  /* Find the last line we wrote on for that note. */
  for (i=0; i < 14; i++)
  {  
    if (!(strcmp(noteid, debug_line[i].identifier) ) ) {
     /* Since this is going to loop through all 14 possible lines,
         last_line will be correct at the end of the loop. */
      last_line = i;
    }
    if (debug_line[i].used == 0) free_lines++; 
  }
  
  if ( (last_line+1 == 14) || free_lines == 0 ) 
  {
    return 14;  /* If we don't have room, no sense continuing.*/
  }
  /* Now, here is the slightly tricky part.  If the line we want is in
  use, we have to move all other lines down one line. */
  if (debug_line[last_line+1].used == 1)
  {
     for (i=13; i > last_line+1; i--)
     {
        sprintf(debug_line[i].identifier, debug_line[i-1].identifier);
	sprintf(debug_line[i].line, debug_line[i-1].line);
	debug_line[i].used = debug_line[i-1].used;
     }
  }
  debug_line[last_line+1].used = 1;
  sprintf(debug_line[last_line+1].line, blankid);
  return last_line+1;
}

/*The best way to use this is to pass in an identifier for each note.
  This allows you to clear one note without clearing all.  Also, you may
  overwrite a note with the same ID string.
  
  Alternatively, you may use NULL as * noteid.
*/     
void take_notes(char * noteid, char * fmt, ...)
{
  int len, paper, pen, cursor;
  char note [1024];
  va_list args;
  
  va_start(args, fmt);
  vsprintf(note, fmt, args);
  va_end(args);
	
  len = strlen(note);
  
  paper = getnewline(noteid);
  
  if (paper==14)
  { 
    return;
  }
  pen = 0;
  
  /* This will be ugly--we're just going to wrap arbitrarily if we write beyond LINEWIDTH. */
  for (cursor = 0; cursor < len; cursor++)
  {
    debug_line[paper].line[pen++] = note[cursor];
    
    if (pen >= LINEWIDTH)
    {
      pen = 0;
      paper = addline(noteid);/*getnewline(noteid);*/
      if (paper==14)
      {
        return;
      }
    }
  }

}
/* assumes you're handling the opening and closing of the transparent list. */
void read_notes()
{
  float y;
  int z;
  
   y = 60.0f;
  for(z=0; z< 14; z++)
  {
    draw_poly_strf(5.0f, y, 100.0f, 1.0f, 1.0f, 1.0f, 1.0f,debug_line[z].line);
    y = y+24.0f;
  }
}  
void clear_note(char * noteid)
{
   int i;
   for (i=0; i< 14; i++)
   {
      if(!(strcmp(noteid, debug_line[i].identifier) ) )
      sprintf(debug_line[i].identifier, blankid);
      debug_line[i].used = 0;
   }
}
void clear_notes()
{
  int i;
  for (i=0; i<14; i++)
  debug_line[i].used = 0;
}
   
