/* ghettoplay.c
 * Copyright (c)2000-2002 Dan Potter
 * Copyright (c)2001 Thorsten Titze
 * Copyright (c) 2006 - 2009 Josh Sutherland <OneThirty8@aol.com>
 * This file is part of VC/DC, a free MPEG-2 video player.
 *
 * VC/DC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VC/DC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* GhettoPlay: an Ogg Vorbis browser and playback util
   (c)2000-2002 Dan Potter
   (c)2001 Thorsten Titze

   Distributed as an example for libdream 0.7
   Ported up to libdream 0.95
   Ported up to KOS 1.1.x
   Converted to OggVorbis

   Historical note: this is _really_ 2.0. There was an internally
   created Ghetto Play that I made to browse and test various S3Ms
   with early versions of the player. It _really_ deserved the name
   GHETTO Play =). This one, like Ghetto Pong, is a 2.0 that is more
   like "Pimpin' Play" ;-). However, sticking with tradition...   
   
*/

#include <stdio.h>
#include "gp.h"
#include "mpegav.h"
#include "vmu_controls.h"
#include "libvcd.h"

long bitrate;
int menu = 0;

int check_start() {
	MAPLE_FOREACH_BEGIN(MAPLE_FUNC_CONTROLLER, cont_state_t, st)
		if (st->buttons & CONT_START) {
			printf("Pressed start\n");
			return 1;
		}
	MAPLE_FOREACH_END()

	return 0;
}

/* Render the mouse if they have one attached */
static int mx = 320, my = 240;
static int lmx[5] = {320, 320, 320, 320, 320},
	lmy[5] = {240, 240, 240, 240, 240};
void mouse_render() {
	int i;
	int atall = 0;
	
	MAPLE_FOREACH_BEGIN(MAPLE_FUNC_MOUSE, mouse_state_t, st)
		atall = 1;
		if (st->dx || st->dy) {
		
			mx += st->dx;
			my += st->dy;
		
			if (mx < 0) mx = 0;
			if (mx > 640) mx = 640;
			if (my < 0) my = 0;
			if (my > 480) my = 480;

			lmx[0] = mx;
			lmy[0] = my;
		}
	MAPLE_FOREACH_END()

	if (atall) {
		for (i=4; i>0; i--) {
			lmx[i] = lmx[i-1];
			lmy[i] = lmy[i-1];
		}

		draw_poly_mouse(mx, my, 1.0f);
		for (i=1; i<5; i++)
			draw_poly_mouse(lmx[i], lmy[i], 0.8f * (5-i) / 6.0f);
	}
}

/* Update the VMU LCD */
#include "vmu_ghetto.h"
#include "vmu_play.h"
#include "vmu_ghettoplay.h"
static int cycle = 0, phase = 1;
void vmu_lcd_update() {
	switch(cycle) {
		case 0: {	/* Getto/Play/GettoPlay */
			if (phase == 1) {
				vmu_set_icon(vmu_ghetto_xpm);
			} else if (phase == 60) {
				vmu_set_icon(vmu_play_xpm);
			} else if (phase == 120) {
				vmu_set_icon(vmu_ghettoplay_xpm);
			} else if (phase == 180) {
				phase = 0;
				cycle = 1;
			}
		} break;
		case 1: {	/* Flashing */
			if ((phase % 20) == 0) {
				vmu_set_icon(vmu_ghettoplay_xpm);
			} else if ((phase % 20) == 10) {
				vmu_set_icon(NULL);
			}
			
			if (phase == 60) {
				cycle = 0;
				phase = 0;
			}
		} break;
	}
	phase++;
}

extern char curdir[1024];
extern int num_entries;
extern int selected, top;
extern int lst_size;
extern volatile int load_queued;
extern char fpsout[138];

/* Copied in from KOS kernel/arch/dreamcast/hardware/cdrom.c */
#define MAKE_SYSCALL(rs, p1, p2, idx) \
	uint32 *syscall_bc = (uint32*)0x8c0000bc; \
	int (*syscall)() = (int (*)())(*syscall_bc); \
	rs syscall((p1), (p2), 0, (idx));

/* Set disc access mode */
static int gdc_change_data_type(void *param) { MAKE_SYSCALL(return, param, 0, 10); }
		
/* Program entry */
int main(int argc, char **argv) {

  int disc_status, last_disc_status, disc_type, checkvcd, isvcd;
  uint32 params[4];
  
  checkvcd = 0; isvcd = 0;

    /* dh - Make it easier to just quit out */
    cont_btn_callback(0, CONT_START | CONT_A | CONT_B | CONT_X | CONT_Y,
                  (cont_btn_callback_t)arch_exit);

  
	/* Do basic setup */
	pvr_init_defaults();

	/* Setup the mouse/font texture */
	setup_util_texture();

	/* Setup background display */
	bkg_setup();
	while (1) {
	scenestart:
		pvr_wait_ready();
		pvr_scene_begin();
		pvr_list_begin(PVR_LIST_OP_POLY);

		/* Opaque list *************************************/
		bkg_render();

		/* End of opaque list */
		pvr_list_finish();
		pvr_list_begin(PVR_LIST_TR_POLY);

		/* Translucent list ********************************/
		draw_poly_strf(5.0f, 60.0f, 100.0f, 1.0f, 1.0f, 1.0f, 1.0f,
			//"         VC/DC  Pre-0.2 Alpha (C)2006-2008         ");
			//"              VC/DC  0.2  (C)2006-2009             ");
			VCDC_VERSION);
		/* Song menu */
		switch (menu){
		case 0:
		song_menu_render();
		
		if (disc_type==0x20 && checkvcd==0){
		   checkvcd = 1;
		   
		   isvcd = open_vcd();
		   
		   if (isvcd)
		   {
		   
		   vmu_set_icon(vcdcontrols_xpm);
		   
		    /* End of translucent list */
		    pvr_list_finish();

		    /* Finish the frame *******************************/
		    pvr_scene_finish();
		    
		    strcpy(curdir, "/vcd");
		    num_entries = 0;
		    load_queued = 0;
		    
		    Start_Pbc();
		    goto scenestart;
		   }
		   
		   
		  
		  }
		break;
		case 1:
		break;
		
		}


		/* Allow the user to replay a VCD without opening the lid */
		if (isvcd==1)
		{
		 if (check_start())
		 {
		  vmu_set_icon(vcdcontrols_xpm);
		  
		  /* End of translucent list */
		  pvr_list_finish();

		  /* Finish the frame *******************************/
		  pvr_scene_finish();
		
		  Start_Pbc();
		  goto scenestart;
		 }		 
		}
				

		/* End of translucent list */
		pvr_list_finish();

		/* Finish the frame *******************************/
		pvr_scene_finish();
		
		/* Update the VMU LCD */
		vmu_lcd_update();
		
		cdrom_get_status(&disc_status, &disc_type);
		if( (disc_status == 6) || (disc_status == 7) )
		{
		  strcpy(curdir, "/cd");
		  menu = 1;
		  num_entries = 0; checkvcd = 0; load_queued = 0; isvcd = 0;
		  lst_size = 0; selected = 0; top = 0;
		  if (!(disc_status == last_disc_status)) close_vcd();
		  last_disc_status = disc_status;
		}  else if (disc_status == 1){
		  last_disc_status = disc_status;
		  menu = 0;
		}
		
		
	}
	

	/* Stop the sound */
	spu_disable();

	return 0;
}





