/*  pbc.c Copyright (c) Josh Sutherland.
 * Copyright (C) 2008 Josh Sutherland <OneThirty8@aol.com>
 * This file is part of VC/DC, a free MPEG-2 video player.
 *
 * VC/DC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VC/DC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* This code is based on vcdimager-0.7.23 by Herbert Valerio Riedel, 
 * Jens B. Jorgensen, and Rocky Bernstein (http://www.vcdimager.org)
 * as well as info found in the source to the xine-vcdnav plugin by Rocky Bernstein
 * (http://sf.net/projects/xine -- xine-vcdnav was the last, but deprecated,
 * CVS version on or about 24 June, 2006).  libcdio-0.77 was used for several things
 * also (http://www.gnu.org/software/libcdio), although obviously not for
 * the actual reading of Compact Discs on the Dreamcast.
 *
 * Further code is based on information gleaned from the VCDImager user docs--
 *  The Wayback Machine @ www.archive.org is your friend!  I plan to mirror
 * that PDF file and some tools for VCD authoring in the future, most likely at
 * www.storm-studios.net/onethirty8/vcdc.html.  Thanks to that doc, I should be
 * able to author some test VCDs and SVCDs to test extended playback control.
 */
/*
 * This is what's 'under the hood' in the VideoCD Playback Control stuff that can
 * be found in libvcd.c.
 * Generally speaking, the contents of this file should be looked at as 'private,'
 * in that if used in another program, your program will not call any of the functions
 * in here directly.  Of course, as per the GPL, you are free to use this code as you
 * wish, but I think that unless you need to alter code in here to suit a particular
 * need, the functions in libvcd.c will likely be more useful in a program that needs
 * to read a (Super)VideoCD, as this just figures out what sector to read from the disc,
 * reads it, and then shoots the data to the demuxer.
 */


#include <kos.h>
#include <libvcd.h>
#include <mpegav.h> 
#include "vcdbuttons.h"
#include "inttypes.h"
#include "vcd_iso.h"

/* We'll just use the same things as the generic vcd loop in mpeg2play.c in here. */
extern int demux (uint8 * buf, uint8 * end, int flags);
extern void vcd_setup_track();
extern void vcd_close_track();
extern void vcd_reset();
extern void mpa_reset();
extern void vidclock_reset();
extern void pause_movie();
extern int seeking;
extern CDROM_TOC *vcd_toc;
extern uint16 vidclock_get_ptime();

extern void redraw_frame();
extern psd_area_t *highlight;

uint8_t * vcd_buffer = NULL;
  
static int pbc_button;
extern RemoteControl_t * remotecontrol;
extern int warn_invalid_frames; /* The number of frames to display an invalid button warning (an X in a box) */

vcd_sector_t *sector_buf; /* This will help us find the bytes we want in the sector header. */

int pbc_get_button()
{
  /* This gives us the raw 'button number' passed to the onscreen
    numerical button selection box. This is the number we want to
    display. */
  return pbc_button;
}

int pbc_handle_button()
{
/* The easiest way to handle the bsn is to check it here.
   This subtracts the Base Selection Number from the button
   selected, and then adds 1 so that we don't return 0.  Returning
   0 would screw us up, because the smallest BSN should be 1 (I think).*/
return (pbc_button - remotecontrol->bsn) + 1;
}

static int a_pressed;
static int b_pressed;
static int x_pressed;
static int y_pressed;
static int start_pressed;

static int left_pressed;
static int right_pressed;
static int up_pressed;
static int down_pressed;

int remote_mode = 0;
int psd_x_button = 0;

static void check_pointer()
{
int i;
  if (remotecontrol->nos == 0){
   highlight = NULL;
   return;
  }
  for (i = 0; i < remotecontrol->nos; i++)
  {
    if(
     ( ( remotecontrol->x) > (remotecontrol->ExtList->area[i].x1)) &&
     ( ( remotecontrol->x) < (remotecontrol->ExtList->area[i].x2)) &&
     ( ( remotecontrol->y) > (remotecontrol->ExtList->area[i].y1)) &&
     ( ( remotecontrol->y) < (remotecontrol->ExtList->area[i].y2))
     )
     {
       pbc_button = i+remotecontrol->bsn;
       highlight = &remotecontrol->ExtList->area[i];
       psd_x_button = 0;
       return;
     }
  }
  if(
     ( ( remotecontrol->x) > (remotecontrol->ExtList->prev_area.x1)) &&
     ( ( remotecontrol->x) < (remotecontrol->ExtList->prev_area.x2)) &&
     ( ( remotecontrol->y) > (remotecontrol->ExtList->prev_area.y1)) &&
     ( ( remotecontrol->y) < (remotecontrol->ExtList->prev_area.y2))
     )
     {
       psd_x_button = BUTTON_PREV;
       highlight = &remotecontrol->ExtList->prev_area;
     }else if(
     ( ( remotecontrol->x) > (remotecontrol->ExtList->next_area.x1)) &&
     ( ( remotecontrol->x) < (remotecontrol->ExtList->next_area.x2)) &&
     ( ( remotecontrol->y) > (remotecontrol->ExtList->next_area.y1)) &&
     ( ( remotecontrol->y) < (remotecontrol->ExtList->next_area.y2))
     )
     {
       psd_x_button = BUTTON_NEXT;
       highlight = &remotecontrol->ExtList->next_area;
     }else if(
     ( ( remotecontrol->x) > (remotecontrol->ExtList->return_area.x1)) &&
     ( ( remotecontrol->x) < (remotecontrol->ExtList->return_area.x2)) &&
     ( ( remotecontrol->y) > (remotecontrol->ExtList->return_area.y1)) &&
     ( ( remotecontrol->y) < (remotecontrol->ExtList->return_area.y2))
     )
     {
       psd_x_button = BUTTON_RET;
       highlight = &remotecontrol->ExtList->return_area;
     }else if(
     ( ( remotecontrol->x) > (remotecontrol->ExtList->default_area.x1)) &&
     ( ( remotecontrol->x) < (remotecontrol->ExtList->default_area.x2)) &&
     ( ( remotecontrol->y) > (remotecontrol->ExtList->default_area.y1)) &&
     ( ( remotecontrol->y) < (remotecontrol->ExtList->default_area.y2))
     )
     {
       psd_x_button = BUTTON_DEF;
       highlight = &remotecontrol->ExtList->default_area;
     } else {
       psd_x_button = 0;
       highlight = NULL;
     }
  
}

static int check_buttons()  /* Checks all of our buttons */
{
int disc_status, disc_type;
      cdrom_get_status(&disc_status, &disc_type);
		if( (disc_status == 6) || (disc_status == 7) )
		{
		  return BUTTON_STOP;
		}
      MAPLE_FOREACH_BEGIN(MAPLE_FUNC_CONTROLLER, cont_state_t, st)
		if (!(st->buttons & CONT_A)) {
		    a_pressed = 0;
		    
		}
		if (!(st->buttons & CONT_B)) {
		    b_pressed = 0;
		    
		}
		if (!(st->buttons & CONT_X)) {
		    x_pressed = 0;
		    
		}
		if (!(st->buttons & CONT_Y)) {
		    y_pressed = 0;
		    
		}
		if (!(st->buttons & CONT_START)) {
		    start_pressed = 0;
		    
		}
		if (!(st->buttons & CONT_DPAD_LEFT)) {
		    left_pressed = 0;
		}
		if (!(st->buttons & CONT_DPAD_RIGHT)) {
		    right_pressed = 0;
		}
		if (!(st->buttons & CONT_DPAD_UP)) {
		    up_pressed = 0;
		}
		if (!(st->buttons & CONT_DPAD_DOWN)) {
		    down_pressed = 0;
		}
		
	     if (remote_mode == 2)
	     {
	        /* Handle pointer for the Selection Areas (if present). */
		if (st->joyx > 45){
			remotecontrol->x++;
		}
		if (st->joyx < -45){
			remotecontrol->x--;
		}
		if (st->joyy > 45){
			remotecontrol->y++;
		}
		if (st->joyy < -45){
			remotecontrol->y--;
		}
		/* Check again to allow multi-speed movement */
		if (st->joyx > 90){
			remotecontrol->x++;
		}
		if (st->joyx < -90){
			remotecontrol->x--;
		}
		if (st->joyy > 90){
			remotecontrol->y++;
		}
		if (st->joyy < -90){
			remotecontrol->y--;
		}
		/* Check boundaries */
		if (remotecontrol->x > 255) remotecontrol->x = 255;
		if (remotecontrol->x < 0) remotecontrol->x = 0;
		if (remotecontrol->y > 255) remotecontrol->y = 255;
		if (remotecontrol->y < 0) remotecontrol->y = 0;
		check_pointer();
	     }
		if (st->buttons & CONT_B) {
		    if (b_pressed==0)
		    {
		        b_pressed = 1;
			
			if ((remotecontrol->nos == 0) && (remotecontrol->ExtList == NULL))
			{
			  return BUTTON_NO_NUMBERS;
			}
			
			if (remote_mode == 0)
			{
			  pbc_button = 0; /* Reset the remote control button number. */
			  psd_x_button = 0;
			}
			if (remote_mode > 0)
			{
			  highlight = NULL;
			  remote_mode = 0;
			} else {
			  remote_mode = (remotecontrol->ExtList == NULL)? 1 : 2;
			}
	#if 0 /* Let's have 'B' just turn on/off the remote mode, and 'A' enter. */			
			if ( (remote_mode == 0) && (pbc_button > 0)&& (psd_x_button == 0)) /* that is, if we just finished entering a number. */
			{
			  highlight = NULL;
			  return BUTTON_NUMERIC;
			} else if ( (remote_mode == 0) &&  (psd_x_button >=BUTTON_PREV) )
			{
			  highlight = NULL;
			  return psd_x_button;
			}else {
			  return BUTTON_REMOTE;
			}
	#else
			return BUTTON_REMOTE;
	#endif			
		    }
		}
		
		if (st->buttons & CONT_A) {
		    if (a_pressed==0)
		    {
		      /* This will be used to enter a numeric button if remote_mode >=1, or enter a hot-spot click
		         if remote_mode == 2, or pause/unpause/turn off ffwd otherwise. */
		        a_pressed = 1;
			if ( (remote_mode > 0) && (pbc_button > 0) && (psd_x_button == 0) )
			{
			  remote_mode = 0;
			  highlight = NULL;
			  return BUTTON_NUMERIC;
			} else if ( (remote_mode == 2) &&  (psd_x_button >=BUTTON_PREV) )
			{
			  remote_mode = 0;
			  highlight = NULL;
			  return psd_x_button;
			} else {
			return BUTTON_PAUSE;
			}
		    }
		} else if (st->buttons & CONT_Y) {
		    if (y_pressed==0)
		    {
		        y_pressed = 1;
			return BUTTON_STOP;
		    }
		} else if ( (st->joyx > 90) && (remote_mode <=1) ){
			seeking = 1;  /* Start skipping frames like crazy. */
			return BUTTON_FFWD;
		} else if (st->buttons & CONT_DPAD_RIGHT) {
		    if (right_pressed==0)
		    {
		        right_pressed = 1;
			if (remote_mode == 0)
			{
			  return BUTTON_NEXT;
			} else {
			  pbc_button++;
			  if (pbc_button < remotecontrol->bsn)
			  {
			   pbc_button = remotecontrol->bsn;
			  }
			  
			  if (pbc_button >= (remotecontrol->bsn + remotecontrol->nos))
			  {
			    pbc_button = remotecontrol->bsn + remotecontrol->nos - 1;
			  }
			  
			  return BUTTON_REMOTE;
			}
		    }
	      } else if (st->buttons & CONT_DPAD_LEFT) {
		    if (left_pressed==0)
		    {
		        left_pressed = 1;
			if (remote_mode == 0)
			{
			  return BUTTON_PREV;
			} else {
			  pbc_button--;
			  if (pbc_button < remotecontrol->bsn)
			  {
			   pbc_button = 0;
			  }
			  return BUTTON_REMOTE;
			}
		    }
	      } else if (st->buttons & CONT_DPAD_UP) {
		    if (up_pressed==0)
		    {
		      int temp;
		        up_pressed = 1;
			if (remote_mode > 0)
			{
			  
			  pbc_button+=10;
			  if (pbc_button < remotecontrol->bsn)
			  {
			   pbc_button = remotecontrol->bsn;
			  }
			  temp = pbc_button % 10;
			  if (temp)
			  {
			    pbc_button += (10 - temp);
			  }
			  
			  if (pbc_button >= (remotecontrol->bsn + remotecontrol->nos))
			  {
			    pbc_button = remotecontrol->bsn + remotecontrol->nos - 1;
			  }
			  
			  return BUTTON_REMOTE;
			}
		    }
	      } else if (st->buttons & CONT_DPAD_DOWN) {
		    if (down_pressed==0)
		    {
		        down_pressed = 1;
			if (remote_mode > 0)
			{
			  pbc_button-=10;
			  if (pbc_button < remotecontrol->bsn)
			  {
			    pbc_button = 0;
			  }
			  return BUTTON_REMOTE;
			}
		    }
	      } else if (st->buttons & CONT_START) {
		    if (start_pressed==0)
		    {
		        start_pressed = 1;
			return BUTTON_DEF;
		    }
	      } else if (st->buttons & CONT_X) {
		    if (x_pressed==0)
		    {
		        x_pressed = 1;
			return BUTTON_RET;
		    }
	      }
	MAPLE_FOREACH_END()
  return 0;
}

static int validate_button(int button)
{
int pbc_handled_button = 0;

  pbc_handled_button = pbc_get_button();
  
   switch (button)
   {
     case BUTTON_PREV:
       if (!(remotecontrol->prev_ofs == 0xffff))
       return BUTTON_VALID;
     break;
     case BUTTON_NEXT:
       if (!(remotecontrol->next_ofs == 0xffff))
       return BUTTON_VALID;
     break;
     case BUTTON_RET: 
       if (!(remotecontrol->return_ofs == 0xffff))
       return BUTTON_VALID;
     break;
     case BUTTON_DEF: 
       if (!(remotecontrol->default_ofs == 0xffff))
       return BUTTON_VALID;
     break;
     case BUTTON_NUMERIC:  /* We will need to fix this to handle BSN at some point. */
       pbc_handled_button = pbc_handle_button();
       if (pbc_handled_button <= remotecontrol->nos)
       return BUTTON_VALID;
     break;
     case BUTTON_STOP:
        return BUTTON_VALID;
     default:
     break;
   }

/* Any other buttons defined should be invalid, so if we fall through to here we don't have a good
  button. */
 return BUTTON_INVALID;   
}

/* static int handle_autopause(uint8 sm);
   
   If the autopause flag in a sector is set (vcdsector_whichever->sm & (1 << 4), or AUTOPAUSE)
   then we call this function.  Just keep polling the controller and redrawing the last frame
   until we time out or get a direction to go someplace else.  This will return either a valid
   button-press, or 0 if it times out.  */
   
static int handle_autopause(uint8 sm)
{
 uint64_t now, then;
 int control_value;
 uint64_t ms_pause; /* The total pause time in milliseconds. */
 
 if (!(sm & AUTOPAUSE)) { return -1;} /* Check the autopause flag and abort if it's not set. */
 if (!(remotecontrol->atime)) { return -1; } /* If we shouldn't pause, get back to work! */
 
 ms_pause = remotecontrol->atime * 1000; /* Converting seconds to milliseconds is pretty easy. */
 
 then = now = timer_ms_gettime64();
 
 mpegav_pause();
 
 while ( (now - then) <  ms_pause) /* While we haven't timed out... */
 {
   now = timer_ms_gettime64();  /* Check the clock... */
   control_value = check_buttons(); /* See if we get an instruction to do something else... */
   if(validate_button(control_value)) /* Make sure it's a valid instruction... */
   {
     return control_value; /* If so, then leave here to follow that instruction... */
   }
   redraw_frame();
   if (remotecontrol->atime == 255) { then = now; } /* If we have an indefinite pause, then don't time out. */
 }
 
 return 0; /* If we time out, then go back to decoding. */
}

static uint32 too_far = 0;
uint32 cur_sector = 0;

/* static int entry_loop (uint32 start_sector);
   
   This is far simpler than the one in mpeg2play.c because next/prev/whatever will
   just return a value that libvcd will translate into the next offset value. All
   button-presses are checked against the valid buttons defined in our global 
   remote_control_t struct, so assuming that we fill this out correctly before calling
   any of the non-static functions in this module, invalid buttons don't cause the loop to return.
   I'm sure this could be prettier, but it will suffice for now. I think. */

static int entry_loop (uint32 start_sector)
{
    uint8 * end;
    int stop_movie;
    int last_button;
    int i;
 
    stop_movie = 0;
    seeking = 0;
    last_button = 0;
    
    cur_sector = start_sector;
       
    do {
	
	/* We have 16 bytes for sync/header and 2*4 bytes subheader
	  at the beginning of each sector, 2324 bytes of movie, and 4 bytes
	  of EDC at the end.  Hacking together a new vfs module here
	  would work (and I'll probably do it anyway for the sake of
	  having one for when a v0.2 release is ready) but reading the
	  raw sector data makes it easier to get at the sector subheader,
	  which contains info we need if we want to come close to conformance
	  to the VideoCD 2.0 standard.*/
	
	end = vcd_buffer + 2348; /* This is really unnecessary... a leftover from the example
					in the mpeg2dec package. */
	 read_sectors(vcd_buffer, cur_sector, 8);//cdrom_read_sectors(vcd_buffer, cur_sector, 8); 
	cur_sector +=8; 
	
	last_button = -1; /* So we handle the controller the first time through this loop. */
	
	for (i=0; i<8; i++)
	{ 
	  sector_buf = (vcd_sector_t *)(vcd_buffer+(i*2352));
	  
	  if (last_button < 0) last_button = check_buttons();
	  
	  if (last_button)
	  {
	    if (last_button==1)  /* Pressed A (play/pause) */
	    {
	      if (seeking==1) 
	      {  seeking = 0; vidclock_reset();}
	      else
	      { pause_movie(); }
	    } else if (last_button==3)
	    {
	       seeking = 1; mpa_reset(); vidclock_reset();
	    }else if (last_button==BUTTON_REMOTE){
	       redraw_frame();
	    } else { 
	      if (validate_button(last_button))
	      {
	       goto done_entry;
	      }else{
	       warn_invalid_frames = 12;
	      }
	    }
	  }
	
	   /* This should be an indication that we have read to the end 
	      of the desired play-item in a properly-encoded stream, but
	      we are not going to be so foolish as to assume we will be
	      handed a properly-encoded stream all the time, so we just
	      reset stuff here to be safe. */
	      
	   if (demux ( (vcd_buffer+(i*2352) )+24, (vcd_buffer+ (i*2352) )+2348, 0 ) )
	   { 
	    vcd_reset();
	     
	   }
	   
	   /* Handle PlayList ptimes. */
	   if (! (remotecontrol->ptime == 0x0000))
	   {
	    uint16 cur_ptime;
	    cur_ptime = vidclock_get_ptime();
	    if (cur_ptime >= remotecontrol->ptime)
	      goto done_entry;
	   }
	   
	   /* See if the sector we just played was an autopause point. */
	   last_button = handle_autopause(sector_buf->sm);
	   
	   if ( ( (cur_sector - 8)+i) >= too_far)
	   {
	     stop_movie = 1;
	   }
	   if (stop_movie==1) break;
	   	   
	}
	   
	if (cur_sector >= too_far) stop_movie = 1;  
 
    } while ( stop_movie == 0 );
done_entry:   
    if (last_button < 0) last_button = 0;
    
    return last_button;
}


/* Note that the nomenclature here isn't totally accurate--'chapters' refer to entry points, 'tracks' to tracks
   (tracks is pretty accurate, actually), and 'stills' to segment play items, which might not be stills at all. */

/* int vcd_play_chapter(_EntriesVcd_sim *entries_list, uint16 chapter);
   Play from the entry point 'chapter' listed in *entries_list, and keep playing to the end of the track or until
   the user enters a valid request via remote control. */
int vcd_play_chapter(_EntriesVcd_sim *entries_list, uint16 chapter)
{

 int last_button;
 
 warn_invalid_frames = 0;
 
  memset (vcd_toc, 0, sizeof(CDROM_TOC));
   //cdrom_read_toc(vcd_toc, 0);
   read_toc(vcd_toc, 0);
   too_far = (entries_list->track[chapter] < vcd_toc->last) ? TOC_LBA(vcd_toc->entry[entries_list->track[chapter]]) : TOC_LBA(vcd_toc->leadout_sector);
   
   if (too_far > TOC_LBA(vcd_toc->leadout_sector))
   {
     too_far = TOC_LBA(vcd_toc->leadout_sector);
   }
   
    vcd_setup_track();
    last_button = entry_loop(entries_list->ent_lba[chapter]);
    vcd_close_track();
   
 return last_button;
}

/* int vcd_play_track(int track_id);
   Play from the start of track number (track_id - 1), and keep playing to the end of the track or until
   the user enters a valid request via remote control. */
int vcd_play_track(int track_id)
{

 int last_button;
 uint32 track_lba;
 
 warn_invalid_frames = 0;
 
 
  memset (vcd_toc, 0, sizeof(CDROM_TOC));
   //cdrom_read_toc(vcd_toc, 0);
  read_toc(vcd_toc, 0);
  
  track_lba = TOC_LBA(vcd_toc->entry[track_id - 1]);
  
   too_far = (track_id < TOC_TRACK(vcd_toc->last))? TOC_LBA(vcd_toc->entry[track_id]) : TOC_LBA(vcd_toc->leadout_sector);
    
   if (too_far > TOC_LBA(vcd_toc->leadout_sector))
   {
     too_far = TOC_LBA(vcd_toc->leadout_sector);
   }

    vcd_setup_track();
    last_button = entry_loop(track_lba);
    vcd_close_track();

 return last_button;
}

/* int vcd_play_still(int chapter_lba);
   Play the Segment Play Item that begins at 'chapter_lba' and check/update the remote control. */
int vcd_play_still(int chapter_lba)
{

 int i, j, sector;  
 int last_button;
 int cn;
    sector = chapter_lba;

    warn_invalid_frames = 0;
   
    cn = 0;  /* Still images can have a hi-res and a lo-res version.
    		Once we get one, skip over the other. This will be fine
		for audio or motion Segment Play Items, as those should
		both have a channel number of 1, but with diffent coding
		information. */
   
    last_button = -1;
    for (i = 0; i < 15; i++)
    {
      read_sectors(vcd_buffer, sector, 10);//cdrom_read_sectors(vcd_buffer, sector, 10);
      for (j = 0; j < 10; j++)
      {
        if (last_button < 0)
	last_button = check_buttons();
        
	sector_buf = (vcd_sector_t *)(vcd_buffer+(j*2352));
	
      /* Skip over empty sectors--the sectors we want will have a value > 0 for coding information. */
	if (sector_buf->ci > 0) 
	{
	  
	 if (cn == 0) /* See if we have found a still image yet. */
	 { cn = sector_buf->cnum; }/* If not, see if we have now. */
	 if (sector_buf->cnum == cn) /* If so, we only care about the one (lo-res or hi-res) we found first. */
	 {
	   if (demux ( (vcd_buffer+(j*2352) )+24, (vcd_buffer+ (j*2352) )+2348, 0 ) )
	   {
	     goto done_still;
	   }
	 }
        }
	if (last_button)
	{
	  if (last_button==1)  /* Pressed A (play/pause) */
	  {
	    if (seeking==1) 
	    {  seeking = 0; vidclock_reset();}
	    else
	    { pause_movie(); }
	  } else if (last_button==3)
	  {
	     seeking = 1; mpa_reset(); vidclock_reset();
	  } else if (last_button==BUTTON_REMOTE){
	     redraw_frame();
	  }else{ 
	    if (validate_button(last_button))
	    {
	     goto done_still;
	    }else{
	     warn_invalid_frames = 12;
	    }
	  }
	}/* end if (last_button) */
	
            /* Handle PlayList ptimes. */
	   if (! (remotecontrol->ptime == 0x0000))
	   {
	    uint16 cur_ptime;
	    cur_ptime = vidclock_get_ptime();
	    if (cur_ptime >= remotecontrol->ptime)
	      goto done_still;
	   }
	   
	   /* See if the sector we just played was an autopause point. */
	   last_button = handle_autopause(sector_buf->sm);
      }
      	
      sector +=10;
    }

done_still:
   vidclock_reset();

  return last_button; /* If we don't get a valid last_button, we can just ignore the value returned here. */
      
}

int vcd_dummy_poll()
{
    return check_buttons();
}

