/* This file and the associated C file(s) contain a bunch of stuff learned by browsing the
 * libcdio 0.77 source code, which will allow us to read a VCD (hopefully) like a regular CD.
 * At the very least, it will give us access to the PVD of the disc and let us access things
 * like the Volume ID and Publisher ID and such.  This will mostly be stuff I found in
 * iso9660.h in libcdio-0.77. There is also a good bit of information throughout libcdio and
 * vcdimager (in the header files mostly, and in a couple places in the C files) dealing with
 * reading sectors that has been very informative, and allowed me to implement such things as
 * Autopause, detection of empty sectors, and presence of audio and/or video by simply reading
 * a bit or byte contained in the sector subheader.
 *
 * Also, we have an ISO-bridge filesystem (See /vcdlib/fs_isobridge.c) for reading VCD based
 * on fs_iso9660 in KOS.  While reading 2352 bytes/sector is what we have been doing dince the
 * 0.1 release in order to get at the 2324 bytes of user data (and we're continuing to do that
 * in PBC to allow us access to the Mode2 subheader to get information on the stream types,
 * autopause flags, etc,) getting at the ISO filesystem in Track 0 (or Track 1 in VCD-ese) is
 * the only way I could think of to read files in Track 0/1 that don't have pre-defined sector
 * addresses.  I now have the ability to implement Extended Playback Control and get at the
 * Scandata files (SCANDATA.DAT or SEARCH.DAT) so I should be able to implement a rewind feature
 * when playing a VCD as a VCD (with or without interpreting the Play Sequence Descriptors).
 *
 * As a free bonus, as long as the VCD stays in the drive, our entire program has access to the
 * "/vcd" filesystem, allowing us to play the *.DAT files right from the disc should we so desire.
 * I'm not sure why a person would want to do that, and it isn't 100% correct, but it's an option
 * that's there.
 *
 */

#ifndef __VCD_ISO_H
#define __VCD_ISO_H

/* Some defines for submode byte bitmasks--values same as in vcdimager/include/libvcd/sector.h */
#define AUTOPAUSE (1<<4)	/* called SM_TRIG in vcdimager. */
#define FORM_2 (1<<5) 		/* called SM_FORM2 in vcdimager. */

typedef struct {
 uint8 synch[12];	/* Synch bytes. */
 uint8 msf[3];		/* Sector location coded as MSF (BCD-coded!) */
 uint8 mode;		/* mode */
} sector_header_t;
 
typedef struct {
 /* Sector Header */
  sector_header_t hdr; 
  
  /* Subheader */
  uint8 fnum;	/* File Number */
  uint8 cnum;	/* Channel Number */
  uint8 sm;	/* Sumbode */
  uint8 ci;	/* Coding Information */
   /* Looks to be exactly the same as above--see vcdimager-0.7.23/lib/sector.c, lines 229-232. */
  uint8 fnum2;
  uint8 cnum2;
  uint8 sm2;
  uint8 ci2;
  
 /* User Data */  
  char data[2324];
  
 /* EDC */
  char junk[4];  
} vcd_sector_t;

/* Break everything, clear cached blocks. */
int iso_bridge_reset();
 
/* Initialize the file system */
int fs_isobridge_init();

/* De-init the file system */
int fs_isobridge_shutdown(); 

/* We don't allow the user to do much with a bin/cue image, but we can essentially read it as if it's VideoCD disc. */
int fs_bincue_mount(const char* fn);

/* We only allow the mounting of one VCD image at a time, which is why the unmount command takes no arguments. */
int fs_bincue_unmount();

/* Pointer to either cdrom_read_sectors or bincue_read_sectors. */
extern int (* read_sectors)(void *buffer, int sector, int cnt);
extern int (* read_toc)(CDROM_TOC *toc_buffer, int session);
void bincue_set_sector_size(int sect_size);


int vcd_read_toc(CDROM_TOC *toc_buffer, int session);

void bsod();
#endif /* __VCD_ISO_H */